@@ -12,6 +12,7 @@ from deprecated.sphinx import versionadded
 from packaging.version import Version
 
 from coredis._utils import b, clusterdown_wrapper, first_key, nativestr
+from coredis.cache import AbstractCache
 from coredis.commands._key_spec import KeySpec
 from coredis.commands.constants import CommandName
 from coredis.commands.core import CoreCommands
@@ -125,19 +126,21 @@ class ClusterMeta(ABCMeta):
                 if (wrapped := add_runtime_checks(method)) != method:
                     setattr(kls, name, wrapped)
                     method = wrapped
-            if doc_addition:
+            if doc_addition and not hasattr(method, "__cluster_docs"):
 
                 def __w(func: Callable[P, Awaitable[R]]) -> Callable[P, Awaitable[R]]:
                     @functools.wraps(func)
                     async def _w(*a: P.args, **k: P.kwargs) -> R:
                         return await func(*a, **k)
 
-                    _w.__doc__ = f"""{textwrap.dedent(method.__doc__)}
+                    _w.__doc__ = f"""{textwrap.dedent(method.__doc__ or "")}
 {doc_addition}
                 """
                     return _w
 
-                setattr(kls, name, __w(method))
+                wrapped = __w(method)
+                setattr(wrapped, "__cluster_docs", doc_addition)
+                setattr(kls, name, wrapped)
         return kls
 
 
@@ -155,6 +158,9 @@ class RedisMeta(ABCMeta):
         return kls
 
 
+RedisConnectionT = TypeVar("RedisConnectionT", bound="RedisConnection")
+
+
 class RedisConnection:
     encoding: str
     decode_responses: bool
@@ -260,7 +266,7 @@ class RedisConnection:
             decode=decode if decode is None else bool(decode)
         )
 
-    async def initialize(self) -> RedisConnection:
+    async def initialize(self: RedisConnectionT) -> RedisConnectionT:
         await self.connection_pool.initialize()
         return self
 
@@ -293,6 +299,8 @@ class AbstractRedis(
     """
 
     server_version: Optional[Version]
+    protocol_version: Literal[2, 3]
+    cache: Optional[AbstractCache]
 
     async def scan_iter(
         self,
@@ -458,6 +466,7 @@ class Redis(
         client_name: Optional[str] = ...,
         protocol_version: Literal[2, 3] = ...,
         verify_version: bool = ...,
+        cache: Optional[AbstractCache] = ...,
         **_: Any,
     ) -> None:
         ...
@@ -490,6 +499,7 @@ class Redis(
         client_name: Optional[str] = ...,
         protocol_version: Literal[2, 3] = ...,
         verify_version: bool = ...,
+        cache: Optional[AbstractCache] = ...,
         **_: Any,
     ) -> None:
         ...
@@ -521,9 +531,15 @@ class Redis(
         client_name: Optional[str] = None,
         protocol_version: Literal[2, 3] = 2,
         verify_version: bool = True,
+        cache: Optional[AbstractCache] = None,
         **_: Any,
     ) -> None:
         """
+        .. versionadded:: 3.9.0
+           If :paramref:`cache` is provided the client will check & populate
+           the cache for read only commands and invalidate it for commands
+           that could change the key(s) in the request.
+
         .. versionchanged:: 3.5.0
            The :paramref:`verify_version` parameter now defaults to ``True``
 
@@ -562,6 +578,10 @@ class Redis(
          version introduced before executing a command and raises a
          :exc:`CommandNotSupportedError` error if the required version is higher than
          the reported server version
+        :param cache: If provided the cache will be used to avoid requests for read only
+         commands if the client has already requested the data and it hasn't been invalidated.
+         The cache is responsible for any mutations to the keys that happen outside of this client
+
         """
         super().__init__(
             host=host,
@@ -591,6 +611,7 @@ class Redis(
         )
         self._use_lua_lock: Optional[bool] = None
         self.response_callbacks: Dict[bytes, Callable[..., Any]] = {}
+        self.cache = cache
 
     @classmethod
     @overload
@@ -601,6 +622,7 @@ class Redis(
         *,
         decode_responses: Literal[False] = ...,
         protocol_version: Literal[2, 3] = ...,
+        verify_version: bool = ...,
         **kwargs: Any,
     ) -> RedisBytesT:
         ...
@@ -614,6 +636,7 @@ class Redis(
         *,
         decode_responses: Literal[True],
         protocol_version: Literal[2, 3] = ...,
+        verify_version: bool = ...,
         **kwargs: Any,
     ) -> RedisStringT:
         ...
@@ -626,6 +649,7 @@ class Redis(
         *,
         decode_responses: bool = False,
         protocol_version: Literal[2, 3] = 2,
+        verify_version: bool = True,
         **kwargs: Any,
     ) -> RedisT:
         """
@@ -659,6 +683,7 @@ class Redis(
             return cls(
                 decode_responses=True,
                 protocol_version=protocol_version,
+                verify_version=verify_version,
                 connection_pool=ConnectionPool.from_url(
                     url,
                     db=db,
@@ -671,6 +696,7 @@ class Redis(
             return cls(
                 decode_responses=False,
                 protocol_version=protocol_version,
+                verify_version=verify_version,
                 connection_pool=ConnectionPool.from_url(
                     url,
                     db=db,
@@ -680,6 +706,12 @@ class Redis(
                 ),
             )
 
+    async def initialize(self) -> Redis[AnyStr]:
+        await super().initialize()
+        if self.cache:
+            await self.cache.initialize(self)
+        return self
+
     def set_response_callback(
         self, command: StringT, callback: Callable[..., Any]
     ) -> None:
@@ -698,10 +730,20 @@ class Redis(
         **options: Optional[ValueT],
     ) -> R:
         """Executes a command and returns a parsed response"""
-        pool = self.connection_pool
-        connection = await pool.get_connection()
+        await self
 
+        pool = self.connection_pool
+        connection = await pool.get_connection(command, *args)
+        if self.cache and connection.tracking_client_id != self.cache.get_client_id(
+            connection
+        ):
+            self.cache.reset()
+            await connection.update_tracking_client(
+                True, self.cache.get_client_id(connection)
+            )
         try:
+            if self.cache and command not in self.connection_pool.READONLY_COMMANDS:
+                self.cache.reset(*KeySpec.extract_keys((command,) + args))
             await connection.send_command(command, *args)
             if custom_callback := self.response_callbacks.get(command):
                 return custom_callback(  # type: ignore
@@ -822,9 +864,7 @@ class Redis(
 
     async def transaction(
         self,
-        func: Callable[
-            ["coredis.pipeline.Pipeline[AnyStr]"], Coroutine[Any, Any, None]
-        ],
+        func: Callable[["coredis.pipeline.Pipeline[AnyStr]"], Coroutine[Any, Any, Any]],
         *watches: KeyT,
         **kwargs: Any,
     ) -> Optional[Any]:
@@ -928,6 +968,7 @@ class RedisCluster(
         protocol_version: Literal[2, 3] = 2,
         verify_version: bool = True,
         non_atomic_cross_slot: bool = False,
+        cache: Optional[AbstractCache] = None,
         **kwargs: Any,
     ):
         """
@@ -1023,6 +1064,7 @@ class RedisCluster(
             bytes, Callable[..., Any]
         ] = self.__class__.RESULT_CALLBACKS.copy()
         self.non_atomic_cross_slot = non_atomic_cross_slot
+        self.cache = cache
 
     @classmethod
     @overload
@@ -1034,6 +1076,7 @@ class RedisCluster(
         skip_full_coverage_check: bool = ...,
         decode_responses: Literal[False] = ...,
         protocol_version: Literal[2, 3] = ...,
+        verify_version: bool = ...,
         **kwargs: Any,
     ) -> RedisClusterBytesT:
         ...
@@ -1048,6 +1091,7 @@ class RedisCluster(
         skip_full_coverage_check: bool = ...,
         decode_responses: Literal[True],
         protocol_version: Literal[2, 3] = ...,
+        verify_version: bool = ...,
         **kwargs: Any,
     ) -> RedisClusterStringT:
         ...
@@ -1061,6 +1105,7 @@ class RedisCluster(
         skip_full_coverage_check: bool = False,
         decode_responses: bool = False,
         protocol_version: Literal[2, 3] = 2,
+        verify_version: bool = True,
         **kwargs: Any,
     ) -> RedisClusterT:
         """
@@ -1091,6 +1136,7 @@ class RedisCluster(
             return cls(
                 decode_responses=True,
                 protocol_version=protocol_version,
+                verify_version=verify_version,
                 connection_pool=ClusterConnectionPool.from_url(
                     url,
                     db=db,
@@ -1104,6 +1150,7 @@ class RedisCluster(
             return cls(
                 decode_responses=False,
                 protocol_version=protocol_version,
+                verify_version=verify_version,
                 connection_pool=ClusterConnectionPool.from_url(
                     url,
                     db=db,
@@ -1114,6 +1161,13 @@ class RedisCluster(
                 ),
             )
 
+    async def initialize(self) -> RedisCluster[AnyStr]:
+        await super().initialize()
+        if self.cache:
+            await self.cache.initialize(self)
+        self.refresh_table_asap = False
+        return self
+
     def __repr__(self) -> str:
         servers = list(
             {
@@ -1165,7 +1219,7 @@ class RedisCluster(
 
         if len(args) <= 0:
             raise RedisClusterException(
-                f"No way to dispatch command:{command!r} with args:{args}"
+                f"No way to dispatch command:{str(command)} with args: {args}"
                 " to Redis Cluster. Missing key."
             )
         keys: Tuple[ValueT, ...] = KeySpec.extract_keys(
@@ -1186,13 +1240,13 @@ class RedisCluster(
         slots = {self.connection_pool.nodes.keyslot(key) for key in keys}
         if not slots:
             raise RedisClusterException(
-                f"No way to dispatch command:{command!r} with args:{args}"
+                f"No way to dispatch command:{str(command)} with args: {args}"
                 " to Redis Cluster. Missing key."
             )
 
         if len(slots) != 1:
             raise RedisClusterException(
-                f"{command!r} - all keys must map to the same key slot"
+                f"{str(command)} - all keys must map to the same key slot"
             )
         return slots.pop()
 
@@ -1249,19 +1303,22 @@ class RedisCluster(
         Sends a command to a node in the cluster
         """
         if not self.connection_pool.initialized:
-            await self.connection_pool.initialize()
+            await self
 
         if not command:
             raise RedisClusterException("Unable to determine command to use")
 
         nodes = self.determine_node(command, **kwargs)
-
         if nodes:
-            return await self.execute_command_on_nodes(
-                nodes, command, *args, callback=callback, **kwargs
-            )
-
-        # If set externally we must update it before calling any commands
+            try:
+                return await self.execute_command_on_nodes(
+                    nodes, command, *args, callback=callback, **kwargs
+                )
+            except ClusterDownError:
+                self.connection_pool.disconnect()
+                self.connection_pool.reset()
+                self.refresh_table_asap = True
+                raise
 
         if self.refresh_table_asap:
             await self.connection_pool.nodes.initialize()
@@ -1306,6 +1363,11 @@ class RedisCluster(
                     await self.parse_response(r, decode=kwargs.get("decode"))
                     asking = False
 
+                if self.cache and r.tracking_client_id != self.cache.get_client_id(r):
+                    self.cache.reset()
+                    await r.update_tracking_client(True, self.cache.get_client_id(r))
+                if self.cache and command not in self.connection_pool.READONLY_COMMANDS:
+                    self.cache.reset(*KeySpec.extract_keys((command,) + args))
                 await r.send_command(command, *args)
 
                 return callback(
@@ -1374,9 +1436,19 @@ class RedisCluster(
                         *node_keys,  # type: ignore
                         *args[1 + key_end :],
                     )
+            if self.cache and command not in self.connection_pool.READONLY_COMMANDS:
+                self.cache.reset(*keys)
         for node in nodes:
             connection = self.connection_pool.get_connection_by_node(node)
-
+            if (
+                self.cache
+                and connection.tracking_client_id
+                != self.cache.get_client_id(connection)
+            ):
+                self.cache.reset()
+                await connection.update_tracking_client(
+                    True, self.cache.get_client_id(connection)
+                )
             # copy from redis-py
             try:
                 if node["name"] in node_arg_mapping:
@@ -1400,7 +1472,11 @@ class RedisCluster(
 
                 if not connection.retry_on_timeout and isinstance(e, TimeoutError):
                     raise
-                await connection.send_command(command, *args)
+                try:
+                    await connection.send_command(command, *args)
+                except ConnectionError as err:
+                    # if a retry attempt results in a connection error assume cluster error
+                    raise ClusterDownError(str(err))
                 res[node["name"]] = callback(
                     await self.parse_response(connection, decode=options.get("decode")),
                     version=self.protocol_version,
