@@ -31,6 +31,7 @@ from coredis.typing import (
     Literal,
     Optional,
     ResponseType,
+    Set,
     Type,
     TypeVar,
     Union,
@@ -95,10 +96,14 @@ class RedisSSLContext:
 
 
 class BaseConnection:
+    client_id: Optional[int]
     description: ClassVar[str] = "BaseConnection"
+    locator: ClassVar[str] = ""
     _reader: Optional[StreamReader]
     _writer: Optional[StreamWriter]
     protocol_version: Literal[2, 3]
+    push_messages: asyncio.Queue[ResponseType]
+    tracking_client_id: Optional[int]
 
     def __init__(
         self,
@@ -135,14 +140,21 @@ class BaseConnection:
         self.server_version: Optional[str] = None
         self.loop = loop
         self.client_name = client_name
+        self.client_id = None
         # flag to show if a connection is waiting for response
         self.awaiting_response = False
         self.last_active_at = time.time()
         self.packer = Packer(self.encoding)
+        self.push_messages: asyncio.Queue[ResponseType] = asyncio.Queue()
+        self.tracking_client_id = None
 
     def __repr__(self) -> str:
         return self.description.format(**self._description_args)
 
+    @property
+    def location(self) -> str:
+        return self.locator.format(**self._description_args)
+
     def __del__(self) -> None:
         try:
             self.disconnect()
@@ -212,10 +224,22 @@ class BaseConnection:
                 f"Failed to authenticate: username={self.username} & password={self.password}"
             )
 
+    async def update_tracking_client(
+        self, enabled: bool, client_id: Optional[int] = None
+    ) -> None:
+        if enabled and client_id is not None:
+            await self.send_command(b"CLIENT TRACKING", b"ON", b"REDIRECT", client_id)
+        else:
+            await self.send_command(b"CLIENT TRACKING", b"OFF")
+        if await self.read_response(decode=False) != b"OK":
+            raise ConnectionError("Unable to toggle client tracking")
+        self.tracking_client_id = client_id
+
     async def on_connect(self) -> None:
         self._parser.on_connect(self)
         hello_command_args: List[Union[int, str, bytes]] = [self.protocol_version]
         auth_attempted = False
+
         if self.username or self.password:
             hello_command_args.extend(
                 ["AUTH", self.username or b"default", self.password or b""]
@@ -224,20 +248,39 @@ class BaseConnection:
         try:
             await self.send_command(b"HELLO", *hello_command_args)
             hello_resp = await self.read_response(decode=False)
-
-            if self.protocol_version == 3:
-                resp3 = cast(Dict[bytes, ValueT], hello_resp)
-                if not resp3[b"proto"] == 3:
-                    raise ConnectionError(
-                        f"Unexpected response when negotiating protocol: [{resp3}]"
-                    )
-                self.server_version = nativestr(resp3[b"version"])
+            if isinstance(hello_resp, (list, dict)):
+                if self.protocol_version == 3:
+                    resp3 = cast(Dict[bytes, ValueT], hello_resp)
+                    if not resp3[b"proto"] == 3:
+                        raise ConnectionError(
+                            f"Unexpected response when negotiating protocol: [{resp3}]"
+                        )
+                    self.server_version = nativestr(resp3[b"version"])
+                    self.client_id = int(resp3[b"id"])
+                else:
+                    resp = cast(List[ValueT], hello_resp)
+                    self.server_version = nativestr(resp[3])
+                    self.client_id = int(resp[7])
             else:
-                resp = cast(List[ValueT], hello_resp)
-                self.server_version = nativestr(resp[3])
+                warnings.warn(
+                    (
+                        f"Unexpected response for `HELLO` command: {hello_resp!r}."
+                        " Unable to determine server version and set protocol version."
+                        " Assuming RESP v2"
+                    ),
+                    category=UserWarning,
+                )
+                self.server_version = None
+                self.client_id = None
+                if self.protocol_version == 3:
+                    raise ConnectionError("Unable to set RESP3 protocol version")
         except (UnknownCommandError, AuthenticationRequiredError):
-            self.version = None
+            if self.protocol_version == 3:
+                raise ConnectionError("Unable to set RESP3 protocol version")
+            self.server_version = None
+            self.client_id = None
             auth_attempted = False
+
         if not auth_attempted and (self.username or self.password):
             if self.username and self.password:
                 await self.send_command(b"AUTH", self.username, self.password)
@@ -254,16 +297,21 @@ class BaseConnection:
 
         if self.client_name is not None:
             await self.send_command(b"CLIENT SETNAME", self.client_name)
-
             if await self.read_response(decode=False) != b"OK":
                 raise ConnectionError(f"Failed to set client name: {self.client_name}")
 
         self.last_active_at = time.time()
 
-    async def read_response(self, decode: Optional[bool] = None) -> ResponseType:
+    async def read_response(
+        self,
+        decode: Optional[bool] = None,
+        push_message_types: Optional[Set[bytes]] = None,
+    ) -> ResponseType:
         try:
             response = await exec_with_timeout(
-                self._parser.read_response(decode=decode),
+                self._parser.read_response(
+                    decode=decode, push_message_types=push_message_types
+                ),
                 self._stream_timeout,
             )
             self.last_active_at = time.time()
@@ -324,6 +372,7 @@ class BaseConnection:
 
 class Connection(BaseConnection):
     description: ClassVar[str] = "Connection<host={host},port={port},db={db}>"
+    locator: ClassVar[str] = "host={host},port={port}"
 
     def __init__(
         self,
@@ -399,7 +448,7 @@ class Connection(BaseConnection):
                         sock.setsockopt(socket.SOL_TCP, k, v)
             except (OSError, TypeError):
                 # `socket_keepalive_options` might contain invalid options
-                # causing an error. Do not leave the connection open.
+                # causing an error. Do not leavo999e the connection open.
                 writer.close()
                 raise
         await self.on_connect()
@@ -407,6 +456,7 @@ class Connection(BaseConnection):
 
 class UnixDomainSocketConnection(BaseConnection):
     description: ClassVar[str] = "UnixDomainSocketConnection<path={path},db={db}>"
+    locator: ClassVar[str] = "path={path}"
 
     def __init__(
         self,
@@ -460,6 +510,7 @@ class UnixDomainSocketConnection(BaseConnection):
 class ClusterConnection(Connection):
     "Manages TCP communication to and from a Redis server"
     description: ClassVar[str] = "ClusterConnection<host={host},port={port}>"
+    locator: ClassVar[str] = "host={host},port={port}"
     node: Node
 
     def __init__(
